$def with (form, results, plugin)
<html>
<head>
    <link rel="stylesheet" href="/static/${plugin}.css" media="screen">
<script type="text/javascript" src="/static/mootools-1.2-core.js"></script>
<script type="text/javascript" src="/static/playMp3.js"></script>
</head>
<body>
<form action="index" class="lonelyFormBlock">
   $:form.render()
   <input type="submit" value="Search" />
</form>
    $if results:
        $for uri, res in results:
            <div class="answer">
            <a href="$uri">$res.artist - $res.album - $res.title</a>
            </div>
</body></html>

